@@ -16,10 +16,6 @@ logger = logging.getLogger(__name__)
 _visited_objects = []
 
 
-class InvalidSignatureInDocstring(RuntimeError):
-    pass
-
-
 class DirectoryWalkerGuard(object):
 
     def __init__(self, dirname):
@@ -39,8 +35,9 @@ class DirectoryWalkerGuard(object):
 
 class FunctionSignature(object):
     # When True keeps generation of stubs with invalid signatures found in docstrings
-    # (yes, global variable, blame me)
+    # (yes, global variables, blame me)
     non_stop_mode = False
+    signature_downgrade = True
 
     # Number of invalid signatures found so far
     n_invalid_signatures = 0
@@ -51,20 +48,22 @@ class FunctionSignature(object):
         self.rtype = rtype
 
         if validate:
-            function_def_str = "def {sig.name}({sig.args}) -> {sig.rtype}: pass".format(sig=self)
+            function_def_str = "def {sig.name}({sig.args}) -> {sig.rtype}: ...".format(sig=self)
             try:
                 ast.parse(function_def_str)
             except SyntaxError as e:
                 FunctionSignature.n_invalid_signatures += 1
-                logger.error("Generated stubs signature is degraded to `(*args, **kwargs) -> Any` for")
-                logger.error(e.text.rstrip())
-                logger.error(" " * (e.offset - 1) + "^-- Invalid syntax")
-                self.name = name
-                self.args = "*args, **kwargs"
-                self.rtype = "Any"
-            if FunctionSignature.n_invalid_signatures > 0 and not FunctionSignature.non_stop_mode:
-                raise InvalidSignatureInDocstring()
-
+                if FunctionSignature.signature_downgrade:
+                    self.name = name
+                    self.args = "*args, **kwargs"
+                    self.rtype = "Any"
+                    logger.error("Generated stubs signature is degraded to `(*args, **kwargs) -> Any` for")
+                    lvl = logging.ERROR
+                else:
+                    lvl = logging.WARNING
+                    logger.warning("Ignoring invalid signature:")
+                logger.log(lvl, e.text.rstrip())
+                logger.log(lvl, " " * (e.offset - 1) + "^-- Invalid syntax")
 
     def __eq__(self, other):
         return isinstance(other, FunctionSignature) and (self.name, self.args, self.rtype) == (
@@ -469,7 +468,7 @@ class PropertyStubsGenerator(StubsGenerator):
 
 class ClassStubsGenerator(StubsGenerator):
     ATTRIBUTES_BLACKLIST = ("__class__", "__module__", "__qualname__", "__dict__", "__weakref__", "__annotations__")
-    PYBIND11_ATTRIBUTES_BLACKLIST = ("__entries", )
+    PYBIND11_ATTRIBUTES_BLACKLIST = ("__entries",)
     METHODS_BLACKLIST = ("__dir__", "__sizeof__")
     BASE_CLASS_BLACKLIST = ("pybind11_object", "object")
     CLASS_NAME_BLACKLIST = ("pybind11_type",)
@@ -814,6 +813,8 @@ def main():
                         help="Deprecated.  Use `--root-module-suffix`")
     parser.add_argument("--no-setup-py", action='store_true')
     parser.add_argument("--non-stop", action='store_true', help="Don't stop when encountered invalid signatures")
+    parser.add_argument("--skip-signature-downgrade", action='store_true',
+                        help="Do not downgrade invalid function signatures to func(*args, **kwargs)")
     parser.add_argument("module_names", nargs="+", metavar="MODULE_NAME", type=str, help="modules names")
     parser.add_argument("--log-level", default="WARNING", help="Set output log level")
 
@@ -822,6 +823,9 @@ def main():
     if sys_args.non_stop:
         FunctionSignature.non_stop_mode = True
 
+    if sys_args.skip_signature_downgrade:
+        FunctionSignature.signature_downgrade = False
+
     if sys_args.root_module_suffix_deprecated is not None:
         sys_args.root_module_suffix = sys_args.root_module_suffix_deprecated
         warnings.warn("`--root_module_suffix` is deprecated in favor of `--root-module-suffix`", FutureWarning)
@@ -839,16 +843,18 @@ def main():
 
     if not os.path.exists(output_path):
         os.mkdir(output_path)
-    try:
-        with DirectoryWalkerGuard(output_path):
-            for _module_name in sys_args.module_names:
-                _module = ModuleStubsGenerator(_module_name)
-                _module.parse()
+
+    with DirectoryWalkerGuard(output_path):
+        for _module_name in sys_args.module_names:
+            _module = ModuleStubsGenerator(_module_name)
+            _module.parse()
+            if FunctionSignature.non_stop_mode or FunctionSignature.n_invalid_signatures == 0:
                 _module.stub_suffix = sys_args.root_module_suffix
                 _module.write_setup_py = not sys_args.no_setup_py
                 recursive_mkdir_walker(_module_name.split(".")[:-1], lambda: _module.write())
-    except InvalidSignatureInDocstring:
-        exit(1)
+
+        if FunctionSignature.n_invalid_signatures > 0:
+            exit(1)
 
 
 if __name__ == "__main__":
